"
COPYRIGHT (c) 2020-2023 LabWare

              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libbasic3' }"

"{ NameSpace: Smalltalk }"

Object subclass:#AutomagicSourcePorter
	instanceVariableNames:'source klass klassMap rewriter'
	classVariableNames:''
	poolDictionaries:''
	category:'Kernel-Classes-Support-Autoporting'
!

AutomagicSourcePorter class instanceVariableNames:'defaultKlassMap'

"
 No other class instance variables are inherited by this class.
"
!

!AutomagicSourcePorter class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020-2023 LabWare

              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    This is an abstract base superclass for automagic code porting
    from Smalltalk/X to other dialects. These can be / are used when
    exporting code to be consumed by others.

    [author:]
        Jan Vrany <jan.vrany@labware.com>

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!AutomagicSourcePorter class methodsFor:'instance creation'!

new
    "return an initialized instance"

    ^ self basicNew initialize.
! !

!AutomagicSourcePorter class methodsFor:'accessing'!

defaultKlassMap
    defaultKlassMap isNil ifTrue:[ 
        defaultKlassMap := Dictionary withKeysAndValues: self defaultKlassMapEntries.
    ].
    ^ defaultKlassMap

    "Created: / 01-07-2020 / 12:31:35 / Jan Vrany <jan.vrany@labware.com>"
!

defaultKlassMapEntries
    ^ #()

    "Created: / 01-07-2020 / 12:32:22 / Jan Vrany <jan.vrany@labware.com>"
! !

!AutomagicSourcePorter methodsFor:'accessing'!

klassMap
    ^ klassMap
!

rewriter
    ^ rewriter
! !

!AutomagicSourcePorter methodsFor:'initialization'!

initialize
    super initialize.
    self initializeKlassMap.
    self initializeRewriter

    "Modified: / 29-10-2018 / 15:54:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 21-09-2022 / 12:28:19 / Jan Vrany <jan.vrany@labware.com>"
!

initializeKlassMap
    klassMap := self class defaultKlassMap copy
!

initializeRewriter
    rewriter := ParseTreeSourceRewriter new.
    rewriter alwaysPerformAllSearches: true.
! !

!AutomagicSourcePorter methodsFor:'porting'!

portClass: aClass source: aString
    "Perform all automatic source code transformation on a method's source
     and return new source with all the necessary modifications."

    source := aString.
    klass := aClass.
    self rewriteClassDefinition.
    ^ source

    "Created: / 24-10-2018 / 11:06:31 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

portMethod: aMethod source: aString
    "Perform all automatic source code transformation on a method's source
     and return new source with all the necessary modifications."

    source := aString.
    klass := aMethod mclass.
    self rewrite.
    ^ source

    "Created: / 24-10-2018 / 11:05:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !


!AutomagicSourcePorter methodsFor:'private'!

collectClassRewritesFrom: aCollection into: aDictionary
    "Collect all renames for class names in `aCollection` and
     add an entru for each class that has to be renamed in
     (output) `aDictionary`.

     The renames are collected as folllows:

       (i) If there's a explicit rename rule in `klassMap` use that one.
      (ii) If not, strip namespace prefix (for Bee does not like Smalltalk/X style 
           namespace prefixes)
    "

    aCollection do:[:name |
        |idx nsName clsName|

        (klassMap includesKey:name) ifTrue:[
            "/ (i) an explicit rename...
            aDictionary at:name put:(klassMap at:name).
        ] ifFalse:[
            (name includes:$:) ifTrue:[
                "/ (ii) a class with namespace prefix...
                idx := name indexOfSubCollection:'::'.   
                nsName := name copyTo:idx-1.
                clsName := name copyFrom:idx+2.
                aDictionary at:name put:clsName.
            ]
        ].
    ].

    "Created: / 24-10-2018 / 11:34:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!AutomagicSourcePorter methodsFor:'rewriting'!

rewrite
    self assert: source isString.
    self
        rewriteEOLComments; checkSource;
        rewriteSimpleExpressions; checkSource;
        rewriteClassReferences; checkSource

    "Created: / 16-10-2018 / 20:29:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 05-11-2018 / 14:43:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 22-09-2022 / 11:07:14 / Jan Vrany <jan.vrany@labware.com>"
!

rewriteClassDefinition
    | parser globalsToRename symbolsToRename replacer |

    parser := Parser for: source.
    parser foldConstants:false.
    parser parseExpressionWithSelf: nil
            notifying:nil
            ignoreErrors:false
            ignoreWarnings:false
            inNameSpace: Smalltalk.

    globalsToRename := Dictionary new.
    symbolsToRename := Dictionary new.

    self collectClassRewritesFrom: parser usedGlobals into: globalsToRename.
    self collectClassRewritesFrom: parser usedSymbols into: symbolsToRename.

    (globalsToRename isEmpty and:[symbolsToRename isEmpty]) ifTrue:[^ self ].

    replacer := ParseTreeSourceRewriter new.
    replacer alwaysPerformAllSearches:true.

    globalsToRename keysAndValuesDo:[:oldName :newName |
        replacer replaceVariable: oldName with: newName.
    ].
    symbolsToRename keysAndValuesDo:[:oldName :newName |
        replacer replaceLiteral: oldName with: newName.
    ].

    self rewriteUsingRewriter: replacer

    "Created: / 24-10-2018 / 11:15:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 11-07-2023 / 11:37:34 / Jan Vrany <jan.vrany@labware.com>"
! !

!AutomagicSourcePorter methodsFor:'rewriting-individual'!

rewriteClassReferences

    "By default we strip Smalltalk/X style namespace prefixes
     as few smalltalks out there have namespaces and none using
     C++-style :: namespace separator."
    self rewriteClassReferences: true

    "Created: / 24-10-2018 / 10:17:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 05-11-2018 / 19:31:04 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 11-07-2023 / 11:32:34 / Jan Vrany <jan.vrany@labware.com>"
!

rewriteClassReferences: stripNamespacePrefix
    | globalsToRename symbolsToRename |



    globalsToRename := Dictionary new.
    symbolsToRename := Dictionary new.

    klassMap keysAndValuesDo:[:from :to |
        (source includesString: from) ifTrue: [
            globalsToRename at: from put: to.
            symbolsToRename at: from put: to.
        ].
    ].
    (stripNamespacePrefix and: [source includesString: '::'])ifTrue: [ 
        | parser |

        parser := Parser for: source in: klass.
        parser foldConstants:false.
        parser parseMethod.

        self collectClassRewritesFrom: parser usedGlobals into: globalsToRename.
        self collectClassRewritesFrom: parser usedSymbols into: symbolsToRename.

    ].

    (globalsToRename notEmpty or: [ symbolsToRename notEmpty ]) ifTrue: [
        | replacer |

        replacer := ParseTreeSourceRewriter new.
        replacer alwaysPerformAllSearches:true. 

        globalsToRename keysAndValuesDo:[:oldName :newName |
            replacer replaceVariable: oldName with: newName.
        ].
        symbolsToRename keysAndValuesDo:[:oldName :newName |
            replacer replaceLiteral: oldName with: newName.
        ].    

        self rewriteUsingRewriter: replacer
    ].

    "Created: / 11-07-2023 / 11:31:21 / Jan Vrany <jan.vrany@labware.com>"
!

rewriteEOLComments
    |parser eolComments xlatedComment newSource|

    "/ Short-circuit if there's no EOL-comment in the code...
    (source includesSubString: '"/') ifFalse: [ ^ self ].

    parser := Parser new.
    parser foldConstants:false.
    parser saveComments:true.
    parser parseMethod:source in:klass ignoreErrors:true ignoreWarnings:true.

    eolComments := parser comments select:[:each | each isEndOfLineComment].
    eolComments notEmpty ifTrue:[
        "/ start with the last (so I don't have to update the positions)
        eolComments sort:[:a :b | a startPosition < b startPosition].

        eolComments reverseDo:[:each |
            xlatedComment := source copyFrom:(each startPosition+2) to:each endPosition.
            (xlatedComment includes:$") ifTrue:[
                "/ self halt.
                xlatedComment := xlatedComment copyReplaceAll:$" with:$'.
            ].
            newSource := source copyTo:(each startPosition-1).
            newSource := newSource,'" ',xlatedComment,'"'.
            newSource := newSource,(source copyFrom:(each endPosition + 1)).
            source := newSource.
        ].
    ].

    "Created: / 16-10-2018 / 20:38:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 06-11-2018 / 21:00:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

rewriteSimpleExpressions
    self rewriteUsingRewriter: rewriter

    "Modified: / 22-09-2022 / 11:23:44 / Jan Vrany <jan.vrany@labware.com>"
!

rewriteUsingRewriter: aParseTreeRewriter
    | tree |

    aParseTreeRewriter searches isEmptyOrNil ifTrue: [ ^ self ].
    [
        tree := RBParser parseMethod:source onError: [ :msg :pos | self error:'Failed to parse method at ',pos printString,': ', msg ].
        aParseTreeRewriter executeTree:tree.
        source := aParseTreeRewriter executeReplacementsInSource:source.
        self checkSource
    ] ensure: [ 
        aParseTreeRewriter forgetReplacements
    ].

    "Created: / 22-09-2022 / 11:23:17 / Jan Vrany <jan.vrany@labware.com>"
    "Modified: / 23-03-2023 / 16:50:56 / Jan Vrany <jan.vrany@labware.com>"
! !

!AutomagicSourcePorter methodsFor:'utilities'!

checkSource
    "Check the source is syntactically correct, if not,
     raise an error."

"/    RBParser 
"/        parseMethod:source
"/        onError:[:aString :pos | self error: 'Invalid source: ', aString , ' at ', pos printString].

    Parser
            parseMethod:source
            in:klass
            ignoreErrors:false
            ignoreWarnings:false

    "Created: / 17-10-2018 / 10:43:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 04-05-2021 / 12:18:34 / Jan Vrany <jan.vrany@labware.com>"
!

rewrite: matchPattern with: rewritePattern
    self rewrite: matchPattern with: rewritePattern when: [ :node | true ]

    "Created: / 29-10-2018 / 15:16:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

rewrite: matchPattern with: rewritePattern when: condition
    | replacer tree |

    replacer := ParseTreeSourceRewriter new.
    replacer alwaysPerformAllSearches:true.
    replacer replace: matchPattern with: rewritePattern when: condition.
    tree := RBParser parseMethod:source.
    replacer executeTree: tree.
    source := replacer executeReplacementsInSource:source.

    self checkSource

    "Created: / 29-10-2018 / 15:15:40 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!AutomagicSourcePorter class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

